﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

using MimeKit;

using MailKit.Net.Smtp;
using MailKit.Security;

using Microsoft.Extensions.Options;

namespace Platform.Mail
{
    public class EmailService : IEmailService
    {
        private string _Domain;
        private string _Server;
        private int _Port;
        private string _SenderAddress;
        private string _SenderName;
        private string _Username;
        private string _Password;
        private bool _SSL;

        public EmailService( IOptionsMonitor<EmailSettings> settings )
        {
            settings.OnChange(OnSettingsChanged);
            OnSettingsChanged(settings.CurrentValue, null);
        }

        private void OnSettingsChanged(EmailSettings settings, string name)
        {
            _Domain = settings.Domain;
            _Server = settings.Server;
            _Port = settings.Port;
            _SenderAddress = settings.SenderAddress;
            _SenderName = settings.SenderName;
            _Username = settings.Username;
            _Password = settings.Password;
            _SSL = settings.SSL;
        }

        public async Task SendEmailAsync(string username, string address, string subject, string text, string html )
        {
            // create message
            var builder = new BodyBuilder();
            builder.TextBody = text;
            builder.HtmlBody = html;

            // todo: attachments

            // create mime message
            var message = new MimeMessage();
            message.From.Add( new MailboxAddress(_SenderName, _SenderAddress));
            message.To.Add(new MailboxAddress(username, address));
            message.Subject = subject;
            message.Body = builder.ToMessageBody();

            // create SMTP client
            using (var client = new SmtpClient())
            {
                // set local domain
                client.LocalDomain = _Domain;

                // connect, send and disconnect
                await client.ConnectAsync(_Server, _Port, _SSL ? SecureSocketOptions.Auto : SecureSocketOptions.None);
                await client.SendAsync(message);
                await client.DisconnectAsync(true);
            }
        }
    }
}
